@@ -2,7 +2,8 @@ import functools
 import logging
 from abc import ABC, abstractmethod
 from dataclasses import dataclass
-from typing import Dict, List, Optional, Tuple
+from functools import cached_property
+from typing import List, Optional, Tuple
 
 import requests
 from eth_abi.exceptions import DecodingError
@@ -16,7 +17,6 @@ from .. import EthereumClient, EthereumNetwork
 from ..constants import NULL_ADDRESS
 from ..contracts import (
     get_erc20_contract,
-    get_kyber_network_proxy_contract,
     get_uniswap_factory_contract,
     get_uniswap_v2_factory_contract,
     get_uniswap_v2_pair_contract,
@@ -28,13 +28,9 @@ from .abis.balancer_abis import balancer_pool_abi
 from .abis.cream_abis import cream_ctoken_abi
 from .abis.mooniswap_abis import mooniswap_abi
 from .abis.zerion_abis import ZERION_TOKEN_ADAPTER_ABI
+from .exceptions import CannotGetPriceFromOracle, InvalidPriceFromOracle
 from .helpers.curve_gauge_list import CURVE_GAUGE_TO_LP_TOKEN
-
-try:
-    from functools import cached_property
-except ImportError:
-    from cached_property import cached_property
-
+from .utils import get_decimals
 
 logger = logging.getLogger(__name__)
 
@@ -45,126 +41,45 @@ class UnderlyingToken:
     quantity: int
 
 
-class OracleException(Exception):
-    pass
-
-
-class InvalidPriceFromOracle(OracleException):
-    pass
-
-
-class CannotGetPriceFromOracle(OracleException):
-    pass
-
-
-class PriceOracle(ABC):
+class BaseOracle(ABC):
+    @classmethod
     @abstractmethod
-    def get_price(self, *args) -> float:
-        pass
-
-
-class PricePoolOracle(ABC):
-    @abstractmethod
-    def get_pool_token_price(self, pool_token_address: ChecksumAddress) -> float:
-        pass
-
-
-class UsdPricePoolOracle(ABC):
-    @abstractmethod
-    def get_pool_usd_token_price(self, pool_token_address: ChecksumAddress) -> float:
-        pass
-
-
-class ComposedPriceOracle(ABC):
-    @abstractmethod
-    def get_underlying_tokens(self, *args) -> List[Tuple[UnderlyingToken]]:
-        pass
-
-
-class KyberOracle(PriceOracle):
-    # This is the `tokenAddress` they use for ETH ¯\_(ツ)_/¯
-    ETH_TOKEN_ADDRESS = "0xEeeeeEeeeEeEeeEeEeEeeEEEeeeeEeeeeeeeEEeE"
-    ADDRESSES = {
-        EthereumNetwork.MAINNET: "0x9AAb3f75489902f3a48495025729a0AF77d4b11e",
-        EthereumNetwork.RINKEBY: "0x0d5371e5EE23dec7DF251A8957279629aa79E9C5",
-        EthereumNetwork.ROPSTEN: "0xd719c34261e099Fdb33030ac8909d5788D3039C4",
-        EthereumNetwork.KOVAN: "0xc153eeAD19e0DBbDb3462Dcc2B703cC6D738A37c",
-    }
-
-    def __init__(
-        self,
+    def is_available(
+        cls,
         ethereum_client: EthereumClient,
-        kyber_network_proxy_address: Optional[str] = None,
-    ):
+    ) -> bool:
         """
         :param ethereum_client:
-        :param kyber_network_proxy_address: https://developer.kyber.network/docs/MainnetEnvGuide/#contract-addresses
+        :return: `True` if Oracle is available for the EthereumClient provided, `False` otherwise
         """
-        self.ethereum_client = ethereum_client
-        self.w3 = ethereum_client.w3
-        self._kyber_network_proxy_address = kyber_network_proxy_address
+        raise NotImplementedError
 
-    @cached_property
-    def kyber_network_proxy_address(self):
-        if self._kyber_network_proxy_address:
-            return self._kyber_network_proxy_address
-        return self.ADDRESSES.get(
-            self.ethereum_client.get_network(),
-            self.ADDRESSES.get(EthereumNetwork.MAINNET),
-        )  # By default return Mainnet address
 
-    @cached_property
-    def kyber_network_proxy_contract(self):
-        return get_kyber_network_proxy_contract(
-            self.w3, self.kyber_network_proxy_address
-        )
+class PriceOracle(BaseOracle):
+    @abstractmethod
+    def get_price(self, *args) -> float:
+        raise NotImplementedError
 
-    def get_price(
-        self, token_address_1: str, token_address_2: str = ETH_TOKEN_ADDRESS
-    ) -> float:
-        if token_address_1 == token_address_2:
-            return 1.0
-        try:
-            # Get decimals for token, estimation will be more accurate
-            decimals = self.ethereum_client.erc20.get_decimals(token_address_1)
-            token_unit = int(10**decimals)
-            (
-                expected_rate,
-                _,
-            ) = self.kyber_network_proxy_contract.functions.getExpectedRate(
-                token_address_1, token_address_2, int(token_unit)
-            ).call()
 
-            price = expected_rate / 1e18
+class PricePoolOracle(BaseOracle):
+    @abstractmethod
+    def get_pool_token_price(self, pool_token_address: ChecksumAddress) -> float:
+        raise NotImplementedError
 
-            if price <= 0.0:
-                # Try again the opposite
-                (
-                    expected_rate,
-                    _,
-                ) = self.kyber_network_proxy_contract.functions.getExpectedRate(
-                    token_address_2, token_address_1, int(token_unit)
-                ).call()
-                price = (token_unit / expected_rate) if expected_rate else 0
 
-            if price <= 0.0:
-                error_message = (
-                    f"price={price} <= 0 from kyber-network-proxy={self.kyber_network_proxy_address} "
-                    f"for token-1={token_address_1} to token-2={token_address_2}"
-                )
-                logger.warning(error_message)
-                raise InvalidPriceFromOracle(error_message)
-            return price
-        except (ValueError, BadFunctionCallOutput, DecodingError) as e:
-            error_message = (
-                f"Cannot get price from kyber-network-proxy={self.kyber_network_proxy_address} "
-                f"for token-1={token_address_1} to token-2={token_address_2}"
-            )
-            logger.warning(error_message)
-            raise CannotGetPriceFromOracle(error_message) from e
+class ComposedPriceOracle(BaseOracle):
+    @abstractmethod
+    def get_underlying_tokens(self, *args) -> List[Tuple[UnderlyingToken]]:
+        raise NotImplementedError
 
 
 class UniswapOracle(PriceOracle):
+    """
+    Uniswap V1 Oracle
+
+    https://docs.uniswap.org/protocol/V1/guides/connect-to-uniswap
+    """
+
     ADDRESSES = {
         EthereumNetwork.MAINNET: "0xc0a47dFe034B400B47bDaD5FecDa2621de6c4d95",
         EthereumNetwork.RINKEBY: "0xf5D915570BC477f9B8D6C0E980aA81757A3AaC36",
@@ -186,6 +101,17 @@ class UniswapOracle(PriceOracle):
         self.w3 = ethereum_client.w3
         self._uniswap_factory_address = uniswap_factory_address
 
+    @classmethod
+    def is_available(
+        cls,
+        ethereum_client: EthereumClient,
+    ) -> bool:
+        """
+        :param ethereum_client:
+        :return: `True` if Oracle is available for the EthereumClient provided, `False` otherwise
+        """
+        return ethereum_client.get_network() in cls.ADDRESSES
+
     @cached_property
     def uniswap_factory_address(self):
         if self._uniswap_factory_address:
@@ -199,7 +125,7 @@ class UniswapOracle(PriceOracle):
     def uniswap_factory(self):
         return get_uniswap_factory_contract(self.w3, self.uniswap_factory_address)
 
-    @functools.lru_cache(maxsize=None)
+    @functools.lru_cache(maxsize=2048)
     def get_uniswap_exchange(self, token_address: str) -> str:
         return self.uniswap_factory.functions.getExchange(token_address).call()
 
@@ -303,13 +229,14 @@ class UniswapOracle(PriceOracle):
 
 
 class UniswapV2Oracle(PricePoolOracle, PriceOracle):
+    ROUTER_ADDRESSES = {
+        EthereumNetwork.MAINNET: "0x7a250d5630B4cF539739dF2C5dAcb4c659F2488D",
+    }
+
     # Pair init code is keccak(getCode(UniswapV2Pair))
-    pair_init_code = HexBytes(
+    PAIR_INIT_CODE = HexBytes(
         "0x96e8ac4277198ff8b6f785478aa9a39f403cb768dd02cbee326c3e7da348845f"
     )
-    router_address = (
-        "0x7a250d5630B4cF539739dF2C5dAcb4c659F2488D"  # Same address on every network
-    )
 
     def __init__(
         self, ethereum_client: EthereumClient, router_address: Optional[str] = None
@@ -320,11 +247,29 @@ class UniswapV2Oracle(PricePoolOracle, PriceOracle):
         """
         self.ethereum_client = ethereum_client
         self.w3 = ethereum_client.w3
-        self.router_address: str = router_address or self.router_address
+        self.router_address: str = router_address or self.ROUTER_ADDRESSES.get(
+            ethereum_client.get_network(),
+            self.ROUTER_ADDRESSES[EthereumNetwork.MAINNET],
+        )
         self.router = get_uniswap_v2_router_contract(
             ethereum_client.w3, self.router_address
         )
-        self._decimals_cache: Dict[str, int] = {}
+
+    @classmethod
+    def is_available(
+        cls,
+        ethereum_client: EthereumClient,
+    ) -> bool:
+        """
+        :param ethereum_client:
+        :return: `True` if Oracle is available for the EthereumClient provided, `False` otherwise
+        """
+        return ethereum_client.is_contract(
+            cls.ROUTER_ADDRESSES.get(
+                ethereum_client.get_network(),
+                cls.ROUTER_ADDRESSES[EthereumNetwork.MAINNET],
+            )
+        )
 
     @cached_property
     def factory(self):
@@ -348,7 +293,7 @@ class UniswapV2Oracle(PricePoolOracle, PriceOracle):
         """
         return self.router.functions.WETH().call()
 
-    @functools.lru_cache(maxsize=None)
+    @functools.lru_cache(maxsize=2048)
     def get_pair_address(
         self, token_address: str, token_address_2: str
     ) -> Optional[str]:
@@ -368,6 +313,7 @@ class UniswapV2Oracle(PricePoolOracle, PriceOracle):
             return None
         return pair_address
 
+    @functools.lru_cache(maxsize=2048)
     def calculate_pair_address(self, token_address: str, token_address_2: str):
         """
         Calculate pair address without querying blockchain.
@@ -386,34 +332,17 @@ class UniswapV2Oracle(PricePoolOracle, PriceOracle):
         address = fast_keccak(
             encode_abi_packed(
                 ["bytes", "address", "bytes", "bytes"],
-                [HexBytes("ff"), self.factory_address, salt, self.pair_init_code],
+                [HexBytes("ff"), self.factory_address, salt, self.PAIR_INIT_CODE],
             )
         )[-20:]
         return fast_bytes_to_checksum_address(address)
 
-    def get_decimals(self, token_address: str, token_address_2: str) -> Tuple[int, int]:
-        if not (
-            token_address in self._decimals_cache
-            and token_address_2 in self._decimals_cache
-        ):
-            decimals_1, decimals_2 = self.ethereum_client.batch_call(
-                [
-                    get_erc20_contract(self.w3, token_address).functions.decimals(),
-                    get_erc20_contract(self.w3, token_address_2).functions.decimals(),
-                ]
-            )
-            self._decimals_cache[token_address] = decimals_1
-            self._decimals_cache[token_address_2] = decimals_2
-        return (
-            self._decimals_cache[token_address],
-            self._decimals_cache[token_address_2],
-        )
-
     def get_reserves(self, pair_address: str) -> Tuple[int, int]:
         """
-        Returns the
-        Also returns the block.timestamp (mod 2**32) of the last block during which an interaction occured for the pair.
+        Returns the number of tokens in the pool. `getReserves()` also returns the block.timestamp (mod 2**32) of
+        the last block during which an interaction occurred for the pair, but it's ignored.
         https://uniswap.org/docs/v2/smart-contracts/pair/
+
         :return: Reserves of `token_address` and `token_address_2` used to price trades and distribute liquidity.
         """
         pair_contract = get_uniswap_v2_pair_contract(
@@ -441,7 +370,8 @@ class UniswapV2Oracle(PricePoolOracle, PriceOracle):
             pair_address = self.calculate_pair_address(token_address, token_address_2)
             # Tokens are sorted, so token_1 < token_2
             reserves_1, reserves_2 = self.get_reserves(pair_address)
-            decimals_1, decimals_2 = self.get_decimals(token_address, token_address_2)
+            decimals_1 = get_decimals(token_address, self.ethereum_client)
+            decimals_2 = get_decimals(token_address_2, self.ethereum_client)
             if token_address.lower() > token_address_2.lower():
                 reserves_2, reserves_1 = reserves_1, reserves_2
 
@@ -506,7 +436,8 @@ class UniswapV2Oracle(PricePoolOracle, PriceOracle):
                     pair_contract.functions.totalSupply(),
                 ]
             )
-            decimals_1, decimals_2 = self.get_decimals(token_address_1, token_address_2)
+            decimals_1 = get_decimals(token_address_1, self.ethereum_client)
+            decimals_2 = get_decimals(token_address_2, self.ethereum_client)
 
             # Total value for one token should be the same than total value for the other token
             # if pool is under active arbitrage. We use the price for the first token we find
@@ -534,13 +465,6 @@ class UniswapV2Oracle(PricePoolOracle, PriceOracle):
             raise CannotGetPriceFromOracle(error_message) from e
 
 
-class SushiswapOracle(UniswapV2Oracle):
-    pair_init_code = HexBytes(
-        "0xe18a34eb0e04b04f7a0ac29a6e80748dca96319b42c54d679cb821dca90c6303"
-    )
-    router_address = "0xd9e1cE17f2641f24aE83637ab66a2cca9C378B9F"
-
-
 class AaveOracle(PriceOracle):
     def __init__(self, ethereum_client: EthereumClient, price_oracle: PriceOracle):
         """
@@ -552,6 +476,17 @@ class AaveOracle(PriceOracle):
         self.w3 = ethereum_client.w3
         self.price_oracle = price_oracle
 
+    @classmethod
+    def is_available(
+        cls,
+        ethereum_client: EthereumClient,
+    ) -> bool:
+        """
+        :param ethereum_client:
+        :return: `True` if Oracle is available for the EthereumClient provided, `False` otherwise
+        """
+        return ethereum_client.get_network() == EthereumNetwork.MAINNET
+
     def get_price(self, token_address: str) -> float:
         if (
             token_address == "0x4da27a545c0c5B758a6BA100e3a049001de870f5"
@@ -583,6 +518,17 @@ class CreamOracle(PriceOracle):
         self.w3 = ethereum_client.w3
         self.price_oracle = price_oracle
 
+    @classmethod
+    def is_available(
+        cls,
+        ethereum_client: EthereumClient,
+    ) -> bool:
+        """
+        :param ethereum_client:
+        :return: `True` if Oracle is available for the EthereumClient provided, `False` otherwise
+        """
+        return ethereum_client.get_network() == EthereumNetwork.MAINNET
+
     def get_price(self, token_address: str) -> float:
         try:
             underlying_token = (
@@ -620,6 +566,17 @@ class ZerionComposedOracle(ComposedPriceOracle):
         self.ethereum_client = ethereum_client
         self.w3 = ethereum_client.w3
 
+    @classmethod
+    def is_available(
+        cls,
+        ethereum_client: EthereumClient,
+    ) -> bool:
+        """
+        :param ethereum_client:
+        :return: `True` if Oracle is available for the EthereumClient provided, `False` otherwise
+        """
+        return ethereum_client.get_network() == EthereumNetwork.MAINNET
+
     @cached_property
     def zerion_adapter_contract(self) -> Optional[Contract]:
         """
@@ -742,6 +699,17 @@ class YearnOracle(ComposedPriceOracle):
             ethereum_client, iearn_token_adapter
         )
 
+    @classmethod
+    def is_available(
+        cls,
+        ethereum_client: EthereumClient,
+    ) -> bool:
+        """
+        :param ethereum_client:
+        :return: `True` if Oracle is available for the EthereumClient provided, `False` otherwise
+        """
+        return ethereum_client.get_network() == EthereumNetwork.MAINNET
+
     def get_underlying_tokens(
         self, token_address: ChecksumAddress
     ) -> List[Tuple[float, ChecksumAddress]]:
@@ -777,6 +745,17 @@ class BalancerOracle(PricePoolOracle):
         self.w3 = ethereum_client.w3
         self.price_oracle = price_oracle
 
+    @classmethod
+    def is_available(
+        cls,
+        ethereum_client: EthereumClient,
+    ) -> bool:
+        """
+        :param ethereum_client:
+        :return: `True` if Oracle is available for the EthereumClient provided, `False` otherwise
+        """
+        return ethereum_client.get_network() == EthereumNetwork.MAINNET
+
     def get_pool_token_price(self, pool_token_address: ChecksumAddress) -> float:
         """
         Estimate balancer pool token price based on its components
