@@ -2,6 +2,7 @@ from time import time
 
 from django.test import TestCase
 
+import pytest
 from eth_account import Account
 from web3 import Web3
 
@@ -14,7 +15,8 @@ class TestGnosisProtocolAPI(TestCase):
     @classmethod
     def setUpClass(cls):
         super().setUpClass()
-        cls.gnosis_protocol_api = GnosisProtocolAPI(EthereumNetwork.RINKEBY)
+        cls.mainnet_gnosis_protocol_api = GnosisProtocolAPI(EthereumNetwork.MAINNET)
+        cls.goerli_gnosis_protocol_api = GnosisProtocolAPI(EthereumNetwork.GOERLI)
         cls.gno_token_address = "0x6810e776880C02933D47DB1b9fc05908e5386b96"
         cls.weth_token_address = "0xC02aaA39b223FE8D0A0e5C4F27eAD9083C756Cc2"
         cls.rinkeby_dai_address = "0x5592EC0cfb4dbc12D3aB100b257153436a1f0FEa"
@@ -23,11 +25,13 @@ class TestGnosisProtocolAPI(TestCase):
         random_owner = Account.create().address
         for ethereum_network in (
             EthereumNetwork.MAINNET,
-            EthereumNetwork.RINKEBY,
+            EthereumNetwork.GOERLI,
             EthereumNetwork.XDAI,
         ):
             with self.subTest(ethereum_network=ethereum_network):
-                self.assertEqual(self.gnosis_protocol_api.get_orders(random_owner), [])
+                self.assertEqual(
+                    self.goerli_gnosis_protocol_api.get_orders(random_owner), []
+                )
 
     def test_get_estimated_amount(self):
         gnosis_protocol_api = GnosisProtocolAPI(EthereumNetwork.MAINNET)
@@ -73,26 +77,24 @@ class TestGnosisProtocolAPI(TestCase):
             sellTokenBalance="erc20",
             buyTokenBalance="erc20",
         )
-        self.assertGreaterEqual(self.gnosis_protocol_api.get_fee(order), 0)
+        self.assertGreaterEqual(self.goerli_gnosis_protocol_api.get_fee(order), 0)
 
     def test_get_trades(self):
+        mainnet_order_ui = "0x65F1206182C77A040ED41D507B59C622FA94AB5E71CCA567202CFF3909F3D5C4DBE338E45276630FD8237149DD47EE027AF26F9C619723D0"
         self.assertEqual(
-            self.gnosis_protocol_api.get_trades(
-                "0x9c79b5883b7f2bacbedef554a835fb07c21f4b1b046edf510554a6ba0444d2665ac255889882acd3da2aa939679e3f3d4ce"
-                "a221e72eb7b80"
-            ),
+            self.mainnet_gnosis_protocol_api.get_trades(order_ui=mainnet_order_ui),
             [
                 {
-                    "blockNumber": 9269212,
+                    "blockNumber": 13643462,
                     "logIndex": 0,
-                    "orderUid": "0x9c79b5883b7f2bacbedef554a835fb07c21f4b1b046edf510554a6ba0444d2665ac255889882acd3da2aa939679e3f3d4cea221e72eb7b80",
-                    "buyAmount": "480792",
-                    "sellAmount": "400000000200001",
-                    "sellAmountBeforeFees": "1",
-                    "owner": "0x5ac255889882acd3da2aa939679e3f3d4cea221e",
-                    "buyToken": "0x5592ec0cfb4dbc12d3ab100b257153436a1f0fea",
-                    "sellToken": "0xc778417e063141139fce010982780140aa0cd5ab",
-                    "txHash": "0x4c888ddeac38b195c9ff7220b61df836a49f8fe2fd9a448da2caf56308db1c61",
+                    "orderUid": "0x65f1206182c77a040ed41d507b59c622fa94ab5e71cca567202cff3909f3d5c4dbe338e45276630fd8237149dd47ee027af26f9c619723d0",
+                    "buyAmount": "28361861093850079821",
+                    "sellAmount": "113521821882",
+                    "sellAmountBeforeFees": "113465370931",
+                    "owner": "0xdbe338e45276630fd8237149dd47ee027af26f9c",
+                    "buyToken": "0xc02aaa39b223fe8d0a0e5c4f27ead9083c756cc2",
+                    "sellToken": "0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
+                    "txHash": "0x691d1a8ba39c036e841b6e2ed970f9068ac4a27b61955afb852f11019f2ff4d8",
                 }
             ],
         )
@@ -112,9 +114,11 @@ class TestGnosisProtocolAPI(TestCase):
             sellTokenBalance="erc20",
             buyTokenBalance="erc20",
         )
-        result = self.gnosis_protocol_api.place_order(order, Account().create().key)
+        result = self.goerli_gnosis_protocol_api.place_order(
+            order, Account().create().key
+        )
         self.assertEqual(
-            order["feeAmount"], 0
+            order.feeAmount, 0
         )  # Cannot estimate, as buy token is the same as the sell token
         self.assertEqual(
             result,
@@ -124,12 +128,15 @@ class TestGnosisProtocolAPI(TestCase):
             },
         )
 
-        order["sellToken"] = self.gnosis_protocol_api.weth_address
-        order["buyToken"] = self.rinkeby_dai_address
-        self.assertEqual(
-            self.gnosis_protocol_api.place_order(order, Account().create().key),
-            {
-                "description": "order owner must have funds worth at least x in his account",
-                "errorType": "InsufficientBalance",
-            },
+        order.sellToken = self.goerli_gnosis_protocol_api.weth_address
+        order.buyToken = self.rinkeby_dai_address
+        order_id = self.goerli_gnosis_protocol_api.place_order(
+            order, Account().create().key
         )
+
+        if type(order_id) is dict:
+            if order_id["errorType"] == "NoLiquidity":
+                pytest.xfail("NoLiquidity Error")
+
+        self.assertEqual(order_id[:2], "0x")
+        self.assertEqual(len(order_id), 114)
